/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */
package org.hibara.attachecase;

import java.awt.Toolkit;

/**
 * 各種定数を指定します。
 * 
 * @author Masanori Uozumi
 * @version 2017.01.23<br>
 *          2012.08.28 iniファイルに関する設定を追加<br>
 *          2014.06.18 BLOWFISH_PASS_FOOTERを追加<br>
 *          2016.12.22 バージョン変更<br>
 *          2017.01.23 アタッシェケース#3に対応<br>
 */

public interface AttacheCaseConstant {

	boolean DEBUG = false;
	boolean DEFAULT_MODE = false; // trueでCUIがデフォルトになる

	String INI_FILE_NAME = "attachecase.ini";
	// GUI用
	int SHORT_CUT_KEY = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();
	int V_GAP_FOR_ICON = 40;
	String PROGRESS_BAR_NAME = "progress";
	String CURRENT_FILE_NAME_KEY = "currentfilename";

	String VERSION = "0.73"; // 2017年03月01日

	int ENCRYPT_MODE = 2; // 暗号化処理
	int DECRYPT_MODE = 1; // 復号処理
	int VERSION_MODE = 3;

	int SIZE_OF_LONG = 8; // long型のバイトサイズ
	int SIZE_OF_INT = 4; // int型のバイトサイズ

	long START_OF_DATE = -62135802000000L; // 西暦1年1月1日0時0分のlong値

	// int SLEEP_TIME_FOR_GUI = 20;
	// int SLEEP_TIME_FOR_CUI = 1;

	String BLOWFISH_PASS_FOOTER = "_AttacheCase-M.Hibara"; // Ver1.X用
															// 2014/06/18追加
	String TOKEN_STRING = "_AttacheCaseData"; // 復号の正否に使う
	String DESTROY_TOKEN_STRING = "_Atc_Broken_Data"; // 破壊されているとき
	String ATC_EXTENSION = ".atc"; // 暗号化ファイルの拡張子
	String PASSCODE_ATTACHECASE = "Passcode:AttacheCase\n"; // 暗号部トークン
	String DATE_FORMAT_PATTERN = "yyyy/MM/dd HH:mm:ss";

	// ファイル属性
	int READ_ONLY_FILE_ATTR = 0x01; // リードオンリー
	int ARCHIVE_FILE_ATTR = 0x20; // アーカイブファイル(Windows)

	// アルゴリズムタイプ
	int TYPE_ALGORISM_BLOWFISH = 0; // Blowfish
	int TYPE_ALGORISM_RIJNDAEL = 1; // Rijndael

	// データバージョン
	int OLD_DATA_FILE_VERSION = 103; // Ver1.Xで作成したファイル
	int ATC_DATA_FILE_VERSION = 105;
	int ATC_DATA_FILE_VERSION_3 = 130;
	int ATC_DATA_FILE_VERSION_MAX = 200;

	// データサブバージョン
	int ATC_DATA_SUB_VERSION_6 = 6; // ver.2.70〜
	int ATC_DATA_SUB_VERSION_7 = 7; // ver.2.80〜

	int TYPE_ENCODE_FILE = 0; // 暗号化
	int TYPE_DECODE_FILE = 1; // 復号化

	int Z_MIN_COMPRESSION = 0; // 圧縮率の最小値
	int Z_DEFAULT_COMPRESSION = 6; // デフォルトの圧縮率
	int Z_MAX_COMPRESSION = 9; // 圧縮率の最大値

	int ATTACHE_CASE_APP_VERSION = 3212;

	/*
	 * バッファサイズ
	 */
	int BUF_PASSWORD_SIZE = 264; // 256 + 8（パスワード＋トークン）
	int FILE_BUFFER_SIZE = 1024; // ファイルバッファ

	// #define BUF_SHA1_SIZE 20
	int BUF_SHA1_SIZE = 32; // SHA-1ハッシュサイズ（実際には160bit）

	int READ_FILE_BUF_SIZE = 2048; // ファイル読み込みバッファ

	int BUF_SIZE_32 = 32;
	int BUF_SIZE_256 = 256;
	int BUF_HEADER_SIZE = 255;
	int BUF_FOOTER_SIZE = 16;

	int RIJNDAEL_BLOCK_SIZE = 32;

	// リソースファイル
	String RESOURCE = "org.hibara.attachecase.resource";
	// ResourceBundle BUNDLE = ResourceBundle.getBundle(RESOURCE);
	String MAC_INI_FILE_PATH = System.getProperty("user.home") + "/Library/Preferences/attachecase.ini";

	// リソースバンドル用 確認メッセージ
	String MSG_PROCESS_STOPPED = "MSG_PROCESS_STOPPED";
	String MSG_CONFIRM_OPEN_DECRYPTED_FILES = "MSG_CONFIRM_OPEN_DECRYPTED_FILES";
	String MSG_CONFIRM_OVER_WRITE_SAME_DIR = "MSG_CONFIRM_OVER_WRITE_SAME_DIR";
	String MSG_CONFIRM_OVER_WRITE_SAME_FILE = "MSG_CONFIRM_OVER_WRITE_SAME_FILE";
	String MSG_CONFIRM_CONTINUE = "MSG_CONFIRM_CONTINUE";
	String MSG_INPUT_NEW_DIR = "MSG_INPUT_NEW_DIR";
	String MSG_INPUT_NEW_FILE = "MSG_INPUT_NEW_FILE";
	String MSG_INPUT_PASSWORD = "MSG_INPUT_PASSWORD";

	// リソースバンドル用 エラーメッセージ
	String MSG_ERROR_PASSWORD_WRONG = "MSG_ERROR_PASSWORD_WRONG";
	String MSG_ERROR_COMPARE_FILE = "MSG_ERROR_COMPARE_FILE";
	String MSG_ERROR_UPPER_VERSION = "MSG_ERROR_UPPER_VERSION";
	String MSG_ERROR_LOWER_VERSION = "MSG_ERROR_LOWER_VERSION";
	String MSG_ERROR_DISK_SPACE = "MSG_ERROR_DISK_SPACE";
	String MSG_ERROR_CHECKSUM = "MSG_ERROR_CHECKSUM";
	String MSG_ERROR_FILE_DESTORYED = "MSG_ERROR_FILE_DESTORYED";
	String MSG_ERROR_NOT_ENCRYPT_FILE = "MSG_ERROR_NOT_ENCRYPT_FILE";
	String MSG_ERROR_DIR_EXIST = "MSG_ERROR_DIR_EXIST";
	String MSG_ERROR_FILE_EXIST = "MSG_ERROR_FILE_EXIST";
	String MSG_ERROR_NO_DIR_NAME = "MSG_ERROR_NO_DIR_NAME";
	String MSG_ERROR_NO_FILE_NAME = "MSG_ERROR_NO_FILE_NAME";
	String MSG_ERROR_NOT_FILE_EXIST = "MSG_ERROR_NOT_FILE_EXIST";
	String MSG_ERROR_ILLEGAL_PASSWORD = "MSG_ERROR_ILLEGAL_PASSWORD";
	String MSG_ERROR_OUT_FILE_OPEN = "MSG_ERROR_OUT_FILE_OPEN";
	String MSG_ERROR_OUT_FILE_WRITE = "MSG_ERROR_OUT_FILE_WRITE";
}
