/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */
package org.hibara.attachecase;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;

import org.hibara.attachecase.utility.Utility;

/**
 * GUIを用いて確認を行う。
 * 
 * @author maruuo
 * @version 2013.06.04<br>
 *          2012.08.18 メソッド追加<br>
 *          2013.06.04 Linux上でパスワードの入力ができない不具合に対応するため，パスワード入力画面の作成方法を変更<br>
 */
public class DialogConfirm implements Confirm {
	private JFrame parent = null;

	/**
	 * DialogConfirmを構成します。
	 */
	public DialogConfirm() {
		this(null);
	}

	/**
	 * DialogConfirmを構成します。
	 * 
	 * @param parent
	 *            親フレーム
	 */
	public DialogConfirm(JFrame parent) {
		this.parent = parent;
	}

	public JFrame getParent() {
		return parent;
	}

	public void setParent(JFrame parent) {
		this.parent = parent;
	}

	@Override
	public int confirmYesNo(String s, boolean bundle) {
		if (bundle) {
			s = resources.getString(s).trim();
		}
		s = removeNewLineOfEnd(s);
		int result = JOptionPane.showConfirmDialog(parent, s, resources.getString("YES_NO_SELECT_DIALOG"),
				JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE,
				new ImageIcon(getClass().getResource("/icon_help.png")));
		switch (result) {
		case JOptionPane.CLOSED_OPTION:
		case JOptionPane.NO_OPTION:
			result = Confirm.NO;
			break;
		case JOptionPane.YES_OPTION:
			result = Confirm.YES;
			break;
		}
		return result;
	}

	@Override
	public int confirmYesNoAll(String s, boolean bundle) {
		if (bundle) {
			s = resources.getString(s).trim();
		}
		s = removeNewLineOfEnd(s);
		String[] options = { resources.getString("SELECT_OPTION_ALL"), resources.getString("SELECT_OPTION_YES"),
				resources.getString("SELECT_OPTION_NO"), };
		int result = JOptionPane.showOptionDialog(parent, s, resources.getString("YES_NO_ALL_SELECT_DIALOG"),
				JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE,
				new ImageIcon(getClass().getResource("/icon_help.png")), options,
				resources.getString("SELECT_OPTION_ALL"));
		switch (result) {
		case 0:
			result = Confirm.ALL;
			break;
		case 1:
			result = Confirm.YES;
			break;
		default:
			result = Confirm.NO;
		}
		return result;
	}

	@Override
	public int confirmYesNoCancel(String s, boolean bundle) {
		if (bundle) {
			s = resources.getString(s).trim();
		}
		s = removeNewLineOfEnd(s);
		int result = JOptionPane.showConfirmDialog(parent, s, resources.getString("YES_NO_SELECT_DIALOG"),
				JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE,
				new ImageIcon(getClass().getResource("/icon_help.png")));
		switch (result) {
		case JOptionPane.CLOSED_OPTION:
		case JOptionPane.NO_OPTION:
			result = Confirm.NO;
			break;
		case JOptionPane.YES_OPTION:
			result = Confirm.YES;
			break;
		default:
			result = Confirm.CANCEL;
		}
		return result;
	}

	@Override
	public int confirmYesNoAllCancel(String s, boolean bundle) {
		if (bundle) {
			s = resources.getString(s).trim();
		}
		s = removeNewLineOfEnd(s);
		String[] options = { resources.getString("SELECT_OPTION_ALL"), resources.getString("SELECT_OPTION_YES"),
				resources.getString("SELECT_OPTION_NO"), resources.getString("SELECT_OPTION_CANCEL"), };
		int result = JOptionPane.showOptionDialog(parent, s, resources.getString("YES_NO_ALL_SELECT_DIALOG"),
				JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE,
				new ImageIcon(getClass().getResource("/icon_help.png")), options,
				resources.getString("SELECT_OPTION_ALL"));
		switch (result) {
		case 0:
			result = Confirm.ALL;
			break;
		case 1:
			result = Confirm.YES;
			break;
		case 2:
		case JOptionPane.CLOSED_OPTION:
			result = Confirm.NO;
			break;
		default:
			result = Confirm.CANCEL;
		}
		return result;
	}

	@Override
	public String inputPassword(int loop) {
		return inputPassword(null, loop);
	}

	@Override
	public String inputPassword(String fileName, int loop) {
		return inputPassword(fileName, loop, false);
	}

	@Override
	public String inputPassword(String fileName, int loop, boolean showPassword) {
		// System.out.println("password = " + showPassword);
		// ループ回数
		if (loop < 0)
			loop = 1;
		if (loop > 2)
			loop = 2;

		String[] inputPasswords = new String[2]; // 入力されたパスワードを格納する配列
		int count = 0;
		// 確認も含めて2回入力
		// 1回目と2回目では入力欄の背景色を変更して区別できるようにする。
		while (count < loop) {
			StringBuilder sb = new StringBuilder();
			if (fileName != null) {
				sb.append('[').append(fileName).append("] ");
			}
			sb.append(resources.getString((count == 0) ? "MSG_INPUT_PASSWORD" : "MSG_INPUT_PASSWORD_AGAIN"));
			// 文字列の中の":"(コロン)から後の部分を取り除く
			if (sb.indexOf(":") != -1) {
				sb.delete(sb.indexOf(":"), sb.length());
			}
			Object result = null;
			JPanel panel = new JPanel();
			BoxLayout layout = new BoxLayout(panel, BoxLayout.Y_AXIS);
			panel.setLayout(layout);

			JTextField pass = null;
			if (!showPassword) {
				pass = new JPasswordField(20);
			} else {
				pass = new JTextField(20);
			}
			pass.setEditable(true);
			pass.setEnabled(true);
			panel.add(new JLabel(sb.toString()));
			panel.add(pass);
			// コメントアウトしている方法ではLinux上でテキストフィールドに文字の入力ができなかった 2013/06/04
			/*
			 * int select = JOptionPane.showOptionDialog( parent, null,
			 * resources.getString((count == 0) ? "PASSWORD_DIALOG" :
			 * "PASSWORD_CONFIRM_DIALOG"), JOptionPane.OK_CANCEL_OPTION,
			 * JOptionPane.INFORMATION_MESSAGE, new
			 * ImageIcon(getClass().getResource("/user_secured.png")),
			 * panelOptions, panelOptions[1] );
			 */
			/*
			 * int select = JOptionPane.showConfirmDialog( parent, panel,
			 * sb.toString(), JOptionPane.OK_CANCEL_OPTION,
			 * JOptionPane.INFORMATION_MESSAGE, new
			 * ImageIcon(getClass().getResource("/user_secured.png")) );
			 */
			JOptionPane optionPane = new JOptionPane(panel, JOptionPane.INFORMATION_MESSAGE,
					JOptionPane.OK_CANCEL_OPTION, new ImageIcon(getClass().getResource("/user_secured.png"))) {
				private static final long serialVersionUID = 1L;

				@Override
				public void selectInitialValue() {
					// super.selectInitialValue(); //ボタンにフォーカスを当てる
				}
			};
			JDialog dlg = optionPane.createDialog(parent, sb.toString());
			dlg.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
			dlg.setVisible(true);
			Integer select = (Integer) optionPane.getValue();
			if (select == null || select != JOptionPane.OK_OPTION) {
				return null;
			} else {
				result = (!showPassword) ? new String(((JPasswordField) pass).getPassword())
						: ((JTextField) pass).getText();
			}
			if (count == 1 && !inputPasswords[0].equals(result)) {
				// 1回目と2回目が異なる値 エラーメッセージ出力
				showErrorMessageByBundle("MSG_ERROR_PASSWORD_DIFFERENCE");
			} else {
				inputPasswords[count++] = (String) result;
			}
		}
		// パスワードを返す
		return new String(inputPasswords[0]);
	}

	@Override
	public String inputDirName(File f) {
		String path = getFileName(f, removeNewLineOfEnd(resources.getString("MSG_INPUT_NEW_DIR")),
				JFileChooser.DIRECTORIES_ONLY, null);
		return Utility.rejectPathTraversal(path);
	}

	@Override
	public String inputFileName(File f) {
		String path = getFileName(f, removeNewLineOfEnd(resources.getString("MSG_INPUT_NEW_FILE")),
				JFileChooser.FILES_AND_DIRECTORIES, null);
		if (path == null) {
			return null;
		}
		return Utility.rejectPathTraversal(path);
	}

	@Override
	public void showMessageByBundle(String bundle) {
		showErrorMessage(resources.getString(bundle));
	}

	@Override
	public void showErrorMessageByBundle(String bundle) {
		showErrorMessage(resources.getString(bundle));

	}

	@Override
	public void showMessage(String str) {
		str = removeNewLineOfEnd(str);
		JOptionPane.showMessageDialog(parent, str, resources.getString("MESSAGE_DIALOG"),
				JOptionPane.INFORMATION_MESSAGE, new ImageIcon(getClass().getResource("/icon_info.png")));
	}

	@Override
	public void showErrorMessage(String str) {
		str = removeNewLineOfEnd(str);
		JOptionPane.showMessageDialog(parent, str, resources.getString("MESSAGE_DIALOG"), JOptionPane.ERROR_MESSAGE,
				new ImageIcon(getClass().getResource("/icon_warning.png")));
	}

	/**
	 * JFileChooserダイアログを用いて一つのファイルを選択します。
	 * 
	 * @param f
	 *            カレントディレクトリの指定
	 * @param title
	 *            ダイアログのタイトル
	 * @param mode
	 *            選択モードの設定（ファイルのみ，あるいはディレクトリのみ）
	 * @param filter
	 *            フィルタ
	 * @return 選択したファイルの絶対パス
	 */
	public String getFileName(File f, String title, int mode, FileFilter filter) {
		JFileChooser chooser = new JFileChooser();
		if (f != null) {
			chooser.setCurrentDirectory(f);
		}
		if (filter != null) {
			chooser.addChoosableFileFilter(filter);
		}
		chooser.setFileSelectionMode(mode);
		chooser.setDialogTitle(title);
		int result = chooser.showSaveDialog(parent);
		if (result == JFileChooser.CANCEL_OPTION) {
			return null;
		} else {
			return chooser.getSelectedFile().getAbsolutePath();
		}
	}

	/**
	 * JFileChooserダイアログを用いて複数のファイルを選択します。
	 * 
	 * @param f
	 *            カレントディレクトリの指定
	 * @param title
	 *            ダイアログのタイトル
	 * @param mode
	 *            選択モードの設定（ファイルのみ，あるいはディレクトリのみ）
	 * @param filter
	 *            フィルタ
	 * @return 選択したファイルの絶対パスを格納したList。キャンセルの場合は空のList
	 */
	public List<File> getFiles(File f, String title, int mode, FileFilter filter) {
		JFileChooser chooser = new JFileChooser();
		List<File> files = new ArrayList<File>();
		if (f != null) {
			chooser.setCurrentDirectory(f);
		}
		if (filter != null) {
			chooser.addChoosableFileFilter(filter);
		}
		chooser.setFileSelectionMode(mode);
		chooser.setDialogTitle(title);
		chooser.setMultiSelectionEnabled(true);
		int result = chooser.showOpenDialog(parent);
		if (result == JFileChooser.CANCEL_OPTION) {
			return files;
		}
		for (File file : chooser.getSelectedFiles()) {
			files.add(file);
		}
		return files;
	}

	/**
	 * 行末の改行を取り除く
	 * 
	 * @param s
	 * @return 行末の改行を取り除いた文字列
	 */
	private String removeNewLineOfEnd(String s) {
		StringBuilder sb = new StringBuilder(s);
		char ch = sb.charAt(sb.length() - 1);
		while (ch == '\n' || ch == '\r') {
			sb.deleteCharAt(sb.length() - 1);
			ch = sb.charAt(sb.length() - 1);
		}
		return sb.toString();
	}
}
