/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */
package org.hibara.attachecase;

import com.apple.eawt.AppEvent.QuitEvent;
import com.apple.eawt.Application;
import com.apple.eawt.QuitHandler;
import com.apple.eawt.QuitResponse;

/**
 * Mac用終了メニューの設定を行います。
 * 
 * @author Masanori Uozumi
 * @version 2012.08.20
 */
public class GuiForMac {

	/*
	 * インスタンスの生成を抑制するためprivate宣言
	 */
	private GuiForMac() {
		super();
	}

	/**
	 * スクリーンメニューの設定を行います。
	 * 
	 * @param frame
	 *            メインフレーム
	 * @throws NoClassDefFoundError
	 *             クラスが見つからない場合（Mac用のクラス）
	 */
	public static void setUp(final AttacheCase4GUI frame) throws NoClassDefFoundError {
		if (frame == null) {
			throw new IllegalArgumentException();
		}
		try {
			Application app = Application.getApplication();
			// スクリーンメニューやアプリケーションのイベント通知を受け取るリスナーを設定
			app.setQuitHandler(new QuitHandler() {
				@Override
				public void handleQuitRequestWith(QuitEvent evt, QuitResponse response) {
					frame.quit();
					response.performQuit();
				}
			});
		} catch (Exception e) {
			// e.printStackTrace();
		}
	}
}
