/*
 * Copyright (C) 2016-2017 Masanori Uozumi.
 */
package org.hibara.attachecase.entity;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 暗号化されたファイルの情報を管理。
 * 
 * @author Masanori Uozumi
 * @version 2017.01.18<br>
 *          2016.12.22 ファイル情報が重複するバグを修正<br>
 *          2017.01.18 アタッシェケース#3への対応<br>
 */
public class FileInformation {

	protected String header;
	protected String atcFileCreateDateString;
	protected List<FileAttribute> files;
	protected int size = 0;

	/**
	 * コンストラクタ
	 */
	public FileInformation() {
		header = null;
		atcFileCreateDateString = null;
		files = new ArrayList<FileAttribute>();
	}

	/**
	 * コンストラクタ
	 * 
	 * @param list
	 *            ATCファイルのファイル情報（文字列）
	 */
	public FileInformation(List<String> list) {
		try {
			header = list.get(0);
			atcFileCreateDateString = list.get(1);
			files = new ArrayList<FileAttribute>();
			for (int i = 2; i < list.size(); i++) {
				String str = list.get(i);
				if (str.startsWith("Fn_")) {
					FileAttribute attr = new FileAttribute(str);
					files.add(attr);
				}
			}
			if (files.size() == 0) {
				for (int i = 2; i < list.size(); i++) {
					String str = list.get(i);
					if (str.startsWith("U_")) {
						str = new String(str.getBytes("Windows-31J"));
						FileAttribute attr = new FileAttribute(str);
						files.add(attr);
					}
				}
			}
		} catch (Exception e) {
			// 何もしない
		}
	}

	public String getHeader() {
		return header;
	}

	public void setHeader(String header) {
		this.header = header;
	}

	public String getAtcFileCreateDateString() {
		return atcFileCreateDateString;
	}

	public void setAtcFileCreateDateString(String atcFileCreateDateString) {
		this.atcFileCreateDateString = atcFileCreateDateString;
	}

	public List<FileAttribute> getFiles() {
		return files;
	}

	public void setFiles(List<FileAttribute> files) {
		this.files = files;
	}

	public int getSize() {
		return size;
	}

	public void setSize(int size) {
		this.size = size;
	}

	public int getFileListSize() {
		return files.size();
	}

	/**
	 * ATCファイルの作成日付を取得します。
	 * 
	 * @return 作成日付 yyyy/MM/dd HH:mm:ss
	 */
	public long getAtcFileCreateDate() {
		long timestamp = 0;
		SimpleDateFormat df = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
		// 先頭に"LastDateTime:"という文字列が追加されているので，その部分を取り除く
		String dateString = atcFileCreateDateString.substring(atcFileCreateDateString.indexOf(':') + 1);
		try {
			Date date = df.parse(dateString);
			timestamp = date.getTime();
		} catch (ParseException e) {
			// 何もしない
		}
		return timestamp;
	}

	/**
	 * ヘッダに含まれているすべてのファイルのサイズの合計
	 * 
	 * @return ファイルサイズの合計
	 */
	public long getAllTotalSize() {
		long size = 0;
		for (FileAttribute attr : files) {
			if (attr.getFileSize() != -1) {
				size += attr.getFileSize();
			}
		}
		return size;
	}
}
