/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */

package org.hibara.attachecase.exception;

import org.hibara.attachecase.utility.Utility;

/**
 * ディスク容量が足りないことを通知するシグナルを発生させます。
 * 
 * @author Masanori Uozumi
 * @version 2012.07.26<br>
 *          2012.09.01 サイズの表示方法を変更<br>
 */
public class DiskSpaceException extends AttachecaseException {
	private static final long serialVersionUID = 5197248307281916432L;

	private long freeSize;
	private long needSize;

	/**
	 * 必要とされるディスク容量と，実際の空きディスク容量を設定してDiskSpaceExceptionを構築します。
	 * 
	 * @param free
	 *            実際の空き容量
	 * @param need
	 *            必要とされる容量
	 */
	public DiskSpaceException(long free, long need) {
		bundle = MSG_ERROR_DISK_SPACE;
		freeSize = free;
		needSize = need;
	}

	/**
	 * 必要とされるディスク容量と，実際の空きディスク容量を文字列として取得します。
	 */
	public String getErrorMessage() {
		String s = super.getErrorMessage();
		Object[] dataArgs = null;
		if (needSize < 1000) {
			dataArgs = new Object[] { needSize + "バイト" };
		} else if (needSize < 1000000) {
			dataArgs = new Object[] { (int) (needSize / 1000.0 + 0.5) + "Kバイト", new Long(freeSize) };
		} else {
			dataArgs = new Object[] { (int) (needSize / 1000.0 / 1000.0 + 0.5) + "Mバイト", new Long(freeSize) };
		}
		// 現時点では freeSize は表示していない
		return Utility.replacePlaceHolder(s, dataArgs);
	}
}
