/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */
package org.hibara.attachecase.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URL;

import javax.imageio.ImageIO;
import javax.swing.JPanel;

public class IconButton extends JPanel implements MouseListener {
	/**
	 * serialVersionUID
	 */
	private static final long serialVersionUID = 1L;

	private BufferedImage mouseOutImage;
	private BufferedImage mouseOverImage;
	private boolean mouse_enter;

	public IconButton() {
		Dimension dim = new Dimension(48, 48);
		this.setPreferredSize(dim);
		this.setMaximumSize(dim);
		this.setMinimumSize(dim);
		this.setEnabled(true);
		this.addMouseListener(this);
		this.setBackground(Color.WHITE);
		mouse_enter = false;
	}

	public IconButton(String path1, String path2) {
		this();
		try {
			URL url1 = this.getClass().getResource(path1);
			URL url2 = this.getClass().getResource(path2);
			mouseOutImage = ImageIO.read(url1);
			mouseOverImage = ImageIO.read(url2);
			Dimension dim = new Dimension(mouseOutImage.getHeight(), mouseOutImage.getWidth());
			this.setPreferredSize(dim);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void paint(Graphics g) {
		// 白で塗りつぶしてから画像を置き直す
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, this.getWidth(), this.getHeight());

		if (mouse_enter == true) {
			g.drawImage(mouseOverImage, 0, 0, this);
		} else {
			g.drawImage(mouseOutImage, 0, 0, this);
		}
		g.dispose();
	}

	@Override
	public void mouseClicked(MouseEvent arg0) {
		// TODO 自動生成されたメソッド・スタブ

	}

	@Override
	public void mouseEntered(MouseEvent arg0) {
		mouse_enter = true;
		repaint();
	}

	@Override
	public void mouseExited(MouseEvent arg0) {
		mouse_enter = false;
		repaint();
	}

	@Override
	public void mousePressed(MouseEvent arg0) {
		// TODO 自動生成されたメソッド・スタブ

	}

	@Override
	public void mouseReleased(MouseEvent arg0) {
		// TODO 自動生成されたメソッド・スタブ

	}

	public void setDrop(DropTargetAdapter adaptor) {
		if (adaptor == null) {
			return;
		}
		new DropTarget(this, adaptor);
	}
}
