/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */
package org.hibara.attachecase.io;

import org.hibara.attachecase.AttacheCaseConstant;

/**
 * 
 * @author maruuo
 *
 */
public class ByteArrayOutputStream extends java.io.ByteArrayOutputStream {

	private int headerSizePosition = -1;

	/**
	 * Int型のデータ(4バイト)をリトルエンディアンで書き込みます。
	 * 
	 * @param value 書き込むデータ
	 */
	public void writeInt(int value) {
		for (int i = 0; i < AttacheCaseConstant.SIZE_OF_INT; i++) {
			write(value & 0xff);
			value >>>= 8;
		}
	}

	/**
	 * バッファ上の次のデータの位置を取得します。
	 * 
	 * @return 次のデータを格納する要素のインデックス
	 */
	public int getNextPosition() {
		return count;
	}

	/**
	 * 指定した位置にint型のデータをリトルエンディアンで書き込みます。
	 * 
	 * @param value
	 *            書き込むデータ
	 * @param pos
	 *            書き込み位置
	 * @return 書き込みに成功した場合はtrue，失敗した場合はfalse
	 */
	public boolean writeIntToPosition(int value, int pos) {
		if (pos > count - AttacheCaseConstant.SIZE_OF_INT) {
			return false;
		}
		for (int i = 0; i < AttacheCaseConstant.SIZE_OF_INT; i++) {
			buf[pos++] = (byte) (value & 0xff);
			value >>>= 8;
		}
		return true;
	}

	/**
	 * ATCファイルのヘッダーにヘッダーサイズを書き込みます。
	 * 
	 * @param value
	 *            ヘッダーサイズ
	 * @return 書き込みに成功した場合はtrueが返ります。
	 */
	public boolean writeHeaderSize(int value) {
		if (headerSizePosition < 0) {
			headerSizePosition = count;
			writeInt(value);
			return true;
		}
		return writeIntToPosition(value, headerSizePosition);
	}

	public byte[] getBuffer() {
		return buf;
	}
}
